from __future__ import division
import es
import cmdlib
import random
import path
import playerlib
import gamethread
import cfglib
import weaponlib

info = es.AddonInfo()
info.version                    = '1.0.0'
info.core_version               = '1.0.0'
info.default_modules_version    = '1.0.0'
info.date                       = '2009-05-01'
info.author                     = 'Ojii'
info.basename                   = 'random_effects'
info.shortname                  = 're'
info.fullname                   = 'Random Effects When A Player Is Hurt'

es.ServerVar('random_effects_version', info.version).makepublic()

modules = path.path(es.getAddonPath('random_effects')).joinpath('modules')

config = cfglib.AddonCFG(str(es.ServerVar('eventscripts_gamedir')) + '/cfg/random_effects')
config.text("***************************")
config.text("* Random Effects Settings *")
config.text("***************************")
random_effects_percentage = config.cvar("re_percentage",
                                        100.0,
                                        "Percentage of random effects occuring")
config.write()

weapons = []
for weapon in weaponlib.getWeaponList('#all'):
    for tag in ('#pistol','#shotgun','#smg','#rifle','#sniper','#machinegun',
                '#bow','#gun'):
        if tag in weapon.tags:
            weapons.append(weapon)
            break

def load():
    cmdlib.registerServerCommand('re_load', manager._load,
                                 "Load a module for the random effects addon")
    cmdlib.registerServerCommand('re_load_all', manager._loadall,
                                 "Load all modules for the random effects addon")
    config.execute()
    
class Player(playerlib.Player):
    """
    Imho this should be in the normal playerlib anyway!
    """
    def tell(self, text):
        es.tell(self.userid, 'Random Effect: %s' % text)
        
    def freeze(self, seconds):
        self._freeze(1)
        gamethread.delayed(seconds, self.unfreeze)
        
    def unfreeze(self):
        self._freeze(0)
        
    def burn(self, seconds):
        self._burn()
        gamethread.delayed(seconds, self.extinguish)

class RandomnessManager:
    def __init__(self):
        self.effects = {}
        self.loaded  = []
        self.choices = []
        
    def register(self, name, function, probability):
        if callable(function):
            self.effects[name] = function
            for x in range(probability):
                self.choices.append(name)
        else:
            es.dbgmsg(0,
                      "Unable to register random effect '%s': Not callable!" %
                      name)
            
    def random(self, player):
        self.effects[random.choice(self.choices)](player)
        
    def _load(self, args):
        if not len(args):
            es.dbgmsg(0,'re_load requires an argument')
            return
        if not isinstance(args, basestring):
            args = args[0]
        if not args in self.loaded:
            es.load('random_effects/modules/%s' % args)
            self.loaded.append(args)
        
    def _loadall(self, args):
        """
        Load all modules
        """
        for module in map(lambda x: x.basename(), modules.dirs()):
            self._load(module)
            
manager = RandomnessManager()

def register(name, function, probability=50):
    """
    The higher the probability the more often the effect occurs.
    The normal probability is 50.
    Floating point probabilities will be rounded (down).
    Negative probabilities will be ignored.
    """
    manager.register(name, function, probability)
        
def player_hurt(ev):
    weapon = weaponlib.getWeapon(ev['weapon'])
    if weapon in weapons:
        if random.random() <= float(random_effects_percentage) / 100:
            manager.random(Player(ev['userid']))